#!/system/bin/sh
#By Morpheus

# Sync to data in the rare case a device crashes
sync

# Path
BASEDIR=/data/adb/modules/performance_prem
LOG=/storage/emulated/0/performance/morpheus-Ai.log

# Functions
read_file(){
  if [[ -f $1 ]]; then
    if [[ ! -r $1 ]]; then
      chmod +r "$1"
    fi
    cat "$1"
  else
    echo "File $1 not found"
  fi
}
# Function to write value to system files with permission settings
write(){
  if [[ -f $1 ]]; then
    if [[ ! -w $1 ]]; then
      chmod +w "$1"
    fi
    echo "$2" > $1
    chmod +r "$1"
  else
    echo "File $1 not found"
  fi
}

# Function to read and write value to GPU file with permission settings
rw_gpu_file(){
  local filepath=$1
  local value=$2

  if [[ -f $filepath ]]; then
    if [[ ! -w $filepath ]]; then
      chmod +w "$filepath"
    fi
    echo $value | tee $filepath > /dev/null
  else
    echo "File $filepath not found"
  fi
}

enable_thermal_service() {
  # Enable thermal services
  for thermal in $(resetprop | awk -F '[][]' '/thermal/ {print $2}'); do
    if [[ $(resetprop "$thermal") == "stopped" ]]; then
      service "${thermal/init.svc.thermalloadalgod}" start
      service "${thermal/init.svc.vendor-}" start 
      service "${thermal/init.svc.vendor.}" start
      service "${thermal/init.thermal_}" start
      service "${thermal/init.svc.}" start
      sleep 1
      resetprop -n "$thermal" running
    fi
  done

  # Enable thermal zones  
  find /sys/class/thermal -name mode | grep 'thermal_zone' | while read -r zone; do
    zone_content=$(cat "$zone")
    
    if [ "$zone_content" = 'disabled' ]; then
        write "$zone" 'enabled'
    elif [ "$zone_content" = '0' ]; then
        write "$zone" '1'
    fi
done


  # Enable temp nodes
  for dir in /sys/class/thermal/thermal_zone/*; do
    if [[ -d "$dir" ]]; then
      chmod a+r "$dir/temp"
    fi
  done

  # Enable msm_thermal
  find /sys -name enabled | grep msm_thermal | while read -r node; do
    if [[ "$(cat "$node")" == '0' ]]; then
      write "$node" "1"
    fi
  done
  
  # Restore thermal settings to default values
for THERMAL in /sys/class/thermal/thermal_zone*/type; do
  if grep -E "gpu|ddr" "$THERMAL" >/dev/null; then
    for ZONE in "${THERMAL%/*}"/trip_point_*_temp; do
      DEFAULT_TEMP=$(cat "${ZONE%_temp}_trip_temp")
      write "$ZONE" "$DEFAULT_TEMP"
    done
  fi
done

  # Other thermal enables
  write "/proc/sys/kernel/sched_boost" "0"
  resetprop dalvik.vm.dexopt.thermal-cutoff 2
  resetprop sys.thermal.enable true
  resetprop ro.thermal_warmreset true
}
restore_cpu_clock(){
    min_freq=$(read_file "/sys/devices/system/cpu/cpu*/cpufreq/cpuinfo_min_freq")
    cpu_list=( $(ls -d /sys/devices/system/cpu/cpu*/cpufreq) )
    
    for cpu_path in "${cpu_list[@]}"; do
        write "$cpu_path/scaling_min_freq" "$min_freq"
    done
}

downclock_cpu() {
    cpu_list=( $(ls -d /sys/devices/system/cpu/cpu*/cpufreq) )

    for cpu_path in "${cpu_list[@]}"; do
        scaling_available_frequencies=$(read_file "$cpu_path/scaling_available_frequencies")
        frequencies=( $scaling_available_frequencies )

        big_core_freq=0
        little_core_freq=0

        for freq in "${frequencies[@]}"; do
            if [ $freq -gt $big_core_freq ]; then
                big_core_freq=$freq
            fi
        done

        for freq in "${frequencies[@]}"; do
            if [ $freq -lt $big_core_freq ]; then
                little_core_freq=$freq
                break
            fi
        done

        current_freq=$(read_file "${cpu_path%/*}/cpufreq/scaling_max_freq")

        if [ $current_freq -ge $big_core_freq ]; then
            if [ $big_core_freq -gt 2000000 ]; then
                new_big_freq=$((big_core_freq - (big_core_freq * 20 / 100)))
            else
                new_big_freq=$((big_core_freq - (big_core_freq * 10 / 100)))
            fi
            echo "Setting downclocked big core frequency for $cpu_path to $new_big_freq"
            write "${cpu_path%/*}/cpufreq/scaling_max_freq" "$new_big_freq"
        else
            if [ $little_core_freq -eq 1600000 ]; then
                new_little_freq=$((little_core_freq - (little_core_freq * 10 / 100)))
            else
                new_little_freq=$((little_core_freq - (little_core_freq * 5 / 100)))
            fi
            echo "Setting downclocked little core frequency for $cpu_path to $new_little_freq"
            write "${cpu_path%/*}/cpufreq/scaling_max_freq" "$new_little_freq"
        fi

        echo "Big core frequency for $cpu_path: $new_big_freq"
        echo "Little core frequency for $cpu_path: $new_little_freq"
    done
}

# Loop through GPU files
gpu_path="/sys/kernel/gpu/gpu_available_governor"
gpu_governor_path="/sys/kernel/gpu/gpu_governor"
gpu_content=$(cat $gpu_path)

if [[ "$gpu_content" == *"schedutil"* ]]; then
    echo "Setting schedutil governor for GPU"
    rw_gpu_file $gpu_governor_path "schedutil"
else
    echo "schedutil governor not available for GPU"
fi

# Loop through CPU cores
for cpu_path in /sys/devices/system/cpu/cpu*; do
    cpu_core=$(basename "$cpu_path")
    cpu_avail_governors_file="${cpu_path}/cpufreq/scaling_available_governors"
    cpu_governor_file="${cpu_path}/cpufreq/scaling_governor"
    
    cpu_avail_governors=$(read_file "${cpu_avail_governors_file}")
    
    if [[ "$cpu_avail_governors" == *"schedutil"* ]]; then
        echo "Setting schedutil governor for CPU core $cpu_core"
        write "${cpu_governor_file}" "schedutil"
    else
        echo "schedutil governor not available for CPU core $cpu_core"
    fi
done

# restore cpu ctrl
# Melakukan iterasi pada semua file di dalam direktori
for file in /sys/bus/cpu/devices/*; do
    write "$file/core_ctl/busy_down_thres" "50"
    write "$file/core_ctl/busy_up_thres" "75"
    write "$file/core_ctl/offline_delay_ms" "100"
    write "$file/core_ctl/max_cpus" "3"
    write "$file/core_ctl/enable" "0"
    write "$file/cpufreq/schedutil/rtg_boost_freq" "0"
    write "$file/cpufreq/schedutil/up_rate_limit_us" "0"
done


# enable Service
start logd
start tcpdump
start cnss_diag
start statsd
start traced
stop vendor.perfservice

# Disable logs & debuggers
for exception_trace in $(find /proc/sys/ -name exception-trace); do
    write $exception_trace "0"
done

for compat_log in $(find /proc/sys/ -name compat-log); do
    write $compat_log "0"
done

for tracing_on in $(find /proc/sys/ -name tracing_on); do
    write $tracing_on "0"
done

for log_level in $(find /sys/ -name log_level*); do
    write $log_level "0"
done

for debug_mask in $(find /sys/ -name debug_mask); do
    write $debug_mask "0"
done

for debug_level in $(find /sys/ -name debug_level); do
    write $debug_level "0"
done

for log_ue in $(find /sys/ -name *log_ue*); do
    write $log_ue "0"
done

for log_ce in $(find /sys/ -name *log_ce*); do
    write $log_ce "0"
done

for edac_mc_log in $(find /sys/ -name edac_mc_log*); do
    write $edac_mc_log "0"
done

for enable_event_log in $(find /sys/ -name enable_event_log); do
    write $enable_event_log "0"
done

for log_ecn_error in $(find /sys/ -name log_ecn_error); do
    write $log_ecn_error "0"
done

for sec_log in $(find /sys/ -name sec_log*); do
    write $sec_log "0"
done

# Cmd
cmd thermalservice override-status reset > /dev/null 2>&1

# Enable Thermal
enable_thermal_service

# Cpu core control
restore_cpu_clock
downclock_cpu
#stune
write "/dev/stune/schedtune.colocate" "0"
write "/dev/stune/schedtune.schedtune_boost" "0"
write "/dev/stune/rt/schedtune.schedtune_boost" "0"
write "/dev/stune/schedtune.sched_boost_no_override" "0"

# restore cpus system/background
write "/dev/cpuset/system-background/effective_cpus" "0-3"
write "/dev/cpuset/system-background/cpus" "0-3"

# Schedtune top-app boost
write "/dev/stune/top-app/schedtune.boost" "0"

# Disable sched boost
write "/proc/sys/kernel/sched_boost" "0"
write "/sys/devices/system/cpu/cpu_boost/sched_boost_on_input" "0"

# Fs break time
write "/proc/sys/fs/lease-break-time" "50"

# Set balance
setprop morpheus.mode balance
echo " •> Balance mode activated at $(date "+%H:%M:%S")" >> $LOG

# Report
sed -i -E 's/^description=(\[.*][[:space:]]*)?/description=[ ❄️ Balance Mode... ] /g' "$BASEDIR/module.prop"
cmd notification post -S bigtext -t "❄️ Balance Mode..." > /dev/null 2>&1

exit 0