#!/system/bin/sh
#By Morpheus recode/modify by @reljawa

# Sync to data in the rare case a device crashes
sync

# Path
BASEDIR=/data/adb/modules/performance_prem
LOG=/storage/emulated/0/performance/morpheus-Ai.log

# Functions read only
read_file(){
  if [[ -f $1 ]]; then
    if [[ ! -r $1 ]]; then
      chmod +r "$1"
    fi
    cat "$1"
  else
    echo "File $1 not found"
  fi
}
# Function to write value to system files with permission settings
write(){
  if [[ -f $1 ]]; then
    if [[ ! -w $1 ]]; then
      chmod +w "$1"
    fi
    echo "$2" > $1
    chmod +r "$1"
  else
    echo "File $1 not found"
  fi
} 

# Function to read and write value to GPU file with permission settings
rw_gpu_file(){
  local filepath=$1
  local value=$2

  if [[ -f $filepath ]]; then
    if [[ ! -w $filepath ]]; then
      chmod +w "$filepath"
    fi
    echo $value | tee $filepath > /dev/null
    chmod +r "$filepath"
  else
    echo "File $filepath not found"
  fi
}

disable_thermal_service() {
  # Disable thermal services
  for thermal in $(resetprop | awk -F '[][]' '/thermal/ {print $2}'); do
    if [[ $(resetprop "$thermal") == "running" || $(resetprop "$thermal") == "restarting" ]]; then
      service "${thermal/init.svc.thermalloadalgod}" stop
      service "${thermal/init.svc.vendor-}" stop  
      service "${thermal/init.svc.vendor.}" stop
      service "${thermal/init.thermal_}" stop
      service "${thermal/init.svc.}" stop
      sleep 1
      resetprop -n "$thermal" stopped
    fi
  done

  # Disable thermal zones
  find /sys/class/thermal -name mode | grep 'thermal_zone' | while read -r zone; do
    zone_content=$(cat "$zone")
    
    if [ "$zone_content" = 'enabled' ]; then
        write "$zone" 'disabled'
    elif [ "$zone_content" = '1' ]; then
        write "$zone" '0'
    fi
done

  # Disable temp nodes
  for dir in /sys/class/thermal/thermal_zone*; do
    if [[ -d "$dir" ]]; then
      chmod a-r "$dir/temp"
    fi
  done

  # Disable msm_thermal
  find /sys -name enabled | grep msm_thermal | while read -r node; do
    if [[ "$(cat "$node")" == 'Y' || "$(cat "$node")" == '1' ]]; then
      write "$node" "0"
    fi
  done
  
  # Adjust the thermal settings on the GPU
for THERMAL in /sys/class/thermal/thermal_zone*/type; do
  if grep -E "gpu|ddr" "$THERMAL" >/dev/null; then
    for ZONE in "${THERMAL%/*}"/trip_point_*_temp; do
      CURRENT_TEMP=$(cat "$ZONE")
      if [ "$CURRENT_TEMP" -lt "90000" ]; then
        write "$ZONE" "95000" 
      fi
    done
  fi
done

  # Other thermal disables
  write "/proc/sys/kernel/sched_boost" "0"
  resetprop dalvik.vm.dexopt.thermal-cutoff 0
  resetprop sys.thermal.enable false
  resetprop ro.thermal_warmreset false
}

# lock speed
lock_cpu_clock(){
    max_freq=$(read_file "/sys/devices/system/cpu/cpu*/cpufreq/cpuinfo_max_freq")
    cpu_list=( $(ls -d /sys/devices/system/cpu/cpu*/cpufreq) )
    
    for cpu_path in "${cpu_list[@]}"; do
        write "$cpu_path/scaling_min_freq" "$max_freq"
    done
}
restore_cpu_clk(){
# Get list of CPU directories
cpu_list=(/sys/devices/system/cpu/cpu*/)

for cpu_path in "${cpu_list[@]}"; do
  max_freq=$(read_file "${cpu_path}cpufreq/cpuinfo_max_freq")
  write "${cpu_path}cpufreq/scaling_max_freq" "$max_freq"
done
}


# Loop through GPU files
gpu_path="/sys/kernel/gpu/gpu_available_governor"
gpu_governor_path="/sys/kernel/gpu/gpu_governor"
gpu_content=$(cat $gpu_path)

if [[ "$gpu_content" == *"performance"* ]]; then
    echo "Setting Performance governor for GPU"
    rw_gpu_file $gpu_governor_path "performance"
else
    echo "Performance governor not available for GPU"
fi

# Loop through CPU cores
for cpu_path in /sys/devices/system/cpu/cpu*; do
    cpu_core=$(basename "$cpu_path")
    cpu_avail_governors_file="${cpu_path}/cpufreq/scaling_available_governors"
    cpu_governor_file="${cpu_path}/cpufreq/scaling_governor"
    
    cpu_avail_governors=$(read_file "${cpu_avail_governors_file}")
    
    if [[ "$cpu_avail_governors" == *"performance"* ]]; then
        echo "Setting performance governor for CPU core $cpu_core"
        write "${cpu_governor_file}" "performance"
    else
        echo "Performance governor not available for CPU core $cpu_core"
    fi
done

write "/sys/devices/system/cpu/cpu_boost/sched_boost_on_input" "1"
   
# cpu ctrl
# Melakukan iterasi pada semua file di dalam direktori
for file in /sys/bus/cpu/devices/*; do
    write "$file/core_ctl/busy_down_thres" "40"
    write "$file/core_ctl/busy_up_thres" "60"
    write "$file/core_ctl/offline_delay_ms" "100"
    write "$file/core_ctl/max_cpus" "3"
    write "$file/core_ctl/enable" "0"
    write "$file/cpufreq/schedutil/rtg_boost_freq" "0"
    write "$file/cpufreq/schedutil/up_rate_limit_us" "0"
done

# Disable logs & debuggers
for exception_trace in $(find /proc/sys/ -name exception-trace); do
    write $exception_trace "0"
done

for compat_log in $(find /proc/sys/ -name compat-log); do
    write $compat_log "0"
done

for tracing_on in $(find /proc/sys/ -name tracing_on); do
    write $tracing_on "0"
done

for log_level in $(find /sys/ -name log_level*); do
    write $log_level "0"
done

for debug_mask in $(find /sys/ -name debug_mask); do
    write $debug_mask "0"
done

for debug_level in $(find /sys/ -name debug_level); do
    write $debug_level "0"
done

for log_ue in $(find /sys/ -name *log_ue*); do
    write $log_ue "0"
done

for log_ce in $(find /sys/ -name *log_ce*); do
    write $log_ce "0"
done

for edac_mc_log in $(find /sys/ -name edac_mc_log*); do
    write $edac_mc_log "0"
done

for enable_event_log in $(find /sys/ -name enable_event_log); do
    write $enable_event_log "0"
done

for log_ecn_error in $(find /sys/ -name log_ecn_error); do
    write $log_ecn_error "0"
done

for sec_log in $(find /sys/ -name sec_log*); do
    write $sec_log "0"
done
# Restore
restore_cpu_clk
# lock cpu freq
lock_cpu_clock
# Disable Thermal
disable_thermal_service

# Disable Service
stop logd
stop tcpdump
stop cnss_diag
stop statsd
stop traced
stop vendor.perfservice

# cmd
    cmd shortcut reset-throttling --user 0 > /dev/null 2>&1
    cmd thermalservice override-status 0 > /dev/null 2>&1
    cmd power set-adaptive-power-saver-enabled false > /dev/null 2>&1
    cmd display ab-logging-disable > /dev/null 2>&1
    cmd display dwb-logging-disable > /dev/null 2>&1
    logcat -c > /dev/null 2>&1
# memory tweaker
write "/dev/memcg/memory.use_hierarchy" "1"
write "/dev/memcg/apps/memory.move_charge_at_immigrate" "1"

# limit system cpu usage
write "/dev/cpuset/system-background/effective_cpus" "0-2"
write "/dev/cpuset/system-background/cpus" "0-2"

# foreground
# no need...

# restirected
write "/dev/cpuset/restricted/effective_cpus" "0-2"
write "/dev/cpuset/restricted/cpus" "0-2"

write "/dev/stune/schedtune.colocate" "1"
write "/dev/stune/schedtune.schedtune_boost" "1"
write "/dev/stune/rt/schedtune.schedtune_boost" "1"
write "/dev/stune/schedtune.sched_boost_no_override" "0"

# Fs break time
write "/proc/sys/fs/lease-break-time" "5"

# Set perf
setprop morpheus.mode performance
echo " •> Performance mode activated at $(date "+%H:%M:%S")" >> $LOG

# Report
sed -i -E 's/^description=(\[.*][[:space:]]*)?/description=[ 🔥 Performance Mode... ] /g' "$BASEDIR/module.prop"
cmd notification post -S bigtext -t  "🔥 Performance Mode..." > /dev/null 2>&1

exit 0