#!/system/bin/sh

# Sync to data in the rare case a device crashes
sync

# Path
BASEDIR=/data/adb/modules/performance_prem
LOG=/storage/emulated/0/performance/morpheus-Ai.log
# Functions
read_file(){
  if [[ -f $1 ]]; then
    if [[ ! -r $1 ]]; then
      chmod +r "$1"
    fi
    cat "$1"
  else
    echo "File $1 not found"
  fi
}
# Function to write value to system files with permission settings
write(){
  if [[ -f $1 ]]; then
    if [[ ! -w $1 ]]; then
      chmod +w "$1"
    fi
    echo "$2" > $1
    chmod +r "$1"
  else
    echo "File $1 not found"
  fi
}
# Screen is off, initiate Save Mode
# Loop through CPU cores
for cpu_path in /sys/devices/system/cpu/cpu*; do
    cpu_core=$(basename "$cpu_path")
    cpu_avail_governors_file="${cpu_path}/cpufreq/scaling_available_governors"
    cpu_governor_file="${cpu_path}/cpufreq/scaling_governor"
    
    cpu_avail_governors=$(read_file "${cpu_avail_governors_file}")
    
    if [[ "$cpu_avail_governors" == *"powersave"* ]]; then
        echo "Setting powersave governor for CPU core $cpu_core"
        write "${cpu_governor_file}" "powersave"
    else
        echo "powersave governor not available for CPU core $cpu_core"
    fi
done
 
#  disable memory tweaker
write "/dev/memcg/memory.use_hierarchy" "0"
write "/dev/memcg/apps/memory.move_charge_at_immigrate" "0"

# disabling some cpu tweak
for file in /sys/bus/cpu/devices/*; do
    write "$file/core_ctl/busy_down_thres" "40"
    write "$file/core_ctl/busy_up_thres" "60"
    write "$file/core_ctl/offline_delay_ms" "100"
    write "$file/core_ctl/max_cpus" "3"
    write "$file/core_ctl/enable" "0"
    write "$file/cpufreq/schedutil/rtg_boost_freq" "0"
    write "$file/cpufreq/schedutil/up_rate_limit_us" "0"
done


# disable for better battery life Service
start logd
stop tcpdump
start cnss_diag
start statsd
start traced
stop vendor.perfservice

# disable sched boost

write "/dev/stune/schedtune.colocate" "0"
write "/dev/stune/schedtune.schedtune_boost" "0"
write "/dev/stune/rt/schedtune.schedtune_boost" "0"
write "/dev/stune/schedtune.sched_boost_no_override" "0"
write "/sys/devices/system/cpu/cpu_boost/sched_boost_on_input" "0"

# Set saver
setprop morpheus.mode powersaver
echo " •> Power Saver mode activated at $(date "+%H:%M:%S")" >> $LOG

# Report
sed -i -E 's/^description=(\[.*][[:space:]]*)?/description=[ 💤 Powersaver Mode... ] /g' "$BASEDIR/module.prop"
cmd notification post -S bigtext -t  "💤 Powersaver Mode..." > /dev/null 2>&1

exit 0