#!/system/bin/sh 
# By Morpheus modify by @reljawa

sleep 1

# Path
BASEDIR=/data/adb/modules/performance_prem
INT=/storage/emulated/0
AGD=$INT/performance
LOG=$AGD/morpheus-Ai.log
MSC=$BASEDIR/script
BAL=$MSC/morp_balance.sh
PERF=$MSC/morp_performance.sh
SAV=$MSC/morp_powersaver.sh

# Check morpheus directory
if [ ! -e $AGD ]; then
  mkdir $AGD
fi

idle_mode() {
idle=(
# Idle mode (doze extreme)
"enable"
"force-idle"
"force-inactive"
"step"
"pre-idle-factor 1"
)

for idle in "${idle[@]}"; do
    dumpsys deviceidle $idle
done

am idle-maintenance
# write function
write(){
  if [[ -f $1 ]]; then
    if [[ ! -w $1 ]]; then
      chmod +w "$1"
    fi
    echo "$2" > $1
    chmod +r "$1"
  else
    echo "File $1 not found"
  fi
}
# sett idle mode
globals=(
"device_idle_constants light_after_inactive_to=0,light_pre_idle_to=5000,light_idle_to=3600000,light_max_idle_to=43200000,locating_to=5000,location_accuracy=1000,inactive_to=0,sensing_to=0,motion_inactive_to=0,idle_after_inactive_to=0,idle_to=21600000,max_idle_to=172800000,quick_doze_delay_to=5000,min_time_to_alarm=300000,deep_idle_to=7200000,deep_max_idle_to=86400000,deep_idle_maintenance_max_interval=86400000,deep_idle_maintenance_min_interval=43200000,deep_still_threshold=0,deep_idle_prefetch=1,deep_idle_prefetch_delay=300000,deep_idle_delay_factor=2,deep_idle_factor=3"
)

for glb in "${globals[@]}"; do
    settings put global $glb     
done
}
idle_mode

echo " Profile Mode:" >> $LOG

# Begin of AI
sed -Ei 's/^description=(\[.*][[:space:]]*)?/description=[ 🤖 Ai is started... ] /g' "$BASEDIR/module.prop"
cmd notification post -S bigtext -t  "🤖 Ai is started..." > /dev/null 2>&1

# Start AI
prev_screen_status=""
setprop morpheus.mode notset
while true; do
    sleep 10
    app_list_filter="grep -o -e applist.app.add"
    while IFS= read -r applist || [[ -n "$applist" ]]; do
        filter=$(echo "$applist" | awk '!/ /')
        if [[ -n "$filter" ]]; then
            app_list_filter+=" -e "$filter
        fi
    done < "$AGD/applist_perf.txt"
    window=$(dumpsys window | grep package | $app_list_filter | tail -1)
    if [[ "$window" ]]; then
        PID=$(pidof "$window")
        NUM_CPU=$(nproc)
        AFFINITY_MASK="0x$(printf %0$((NUM_CPU/4))x $(seq 0 $((2**NUM_CPU-1))))"
        if [[ $(getprop morpheus.mode) == "performance" ]]; then
            echo " "
        else
            nice -n -19 $PID
            renice -gpu -n -19 $PID
            ionice -c 2 -n 0 $PID
            chrt -f -p 98 $PID
            taskset -ap $AFFINITY_MASK $PID
            sh $PERF
            write "/proc/$PID/sched_boost" "1"
            write "/proc/$PID/cpuset" "/top-app"
            write "/proc/$PID/sched_low_latency" "1"
        fi
        sleep 1
    else
        if [[ $(getprop morpheus.mode) == "balance" ]]; then
            echo " "
        else
            nice -n 5 $PID
            renice -gpu -n 0 $PID
            ionice -c 3 -n 4 $PID
            chrt -i -p 0 $PID
            sh $BAL
        fi
        sleep 1
   fi
   screen_status=$(dumpsys window | grep "mScreenOn" | grep false)
   if [[ "$screen_status" != "$prev_screen_status" ]]; then
      prev_screen_status="$screen_status"
      if [[ "$screen_status" ]]; then
         if [[ $(getprop morpheus.mode) == "powersaver" ]]; then
            nice -n 10 $PID
            renice -gpu -n 10 $PID
            ionice -c 4 -n 7 $PID
            chrt -i -p 0 $PID
            sh $SAV
            sleep 1
         else
            echo " "
         fi
      fi
   fi
done

# Get total RAM and used RAM
total_ram=$(grep MemTotal /proc/meminfo | awk '{print $2}')
free_ram=$(grep MemAvailable /proc/meminfo | awk '{print $2}')

# Calculate the percentage of RAM used
used_percent=$((($total_ram - $free_ram) * 100 / $total_ram))

echo "Percentage of RAM used: $used_percent%"

# Run the cmd activity kill-all command if the percentage of RAM used is more than 70%
if [ $used_percent -gt 70 ]; then
    echo "The remaining RAM used is excessivei 70%."
    cmd activity kill-all
    echo "The cmd activity kill-all command has been executed."
    cmd notification post -S bigtext -t '[Ram Manager Status]' 'Tag' '🚀 Cleaning complete... ♻️' > /dev/null 2>&1
else
    echo "The remaining RAM used is still within normal limits."
fi